"""
This script is used to access an abstract class for other backends
"""

# pylint: disable=W0311, W0107, R0903
from abc import ABC, abstractmethod
from typing import AsyncIterator, Any
from enum import Enum


class Results(Enum):
	"""This class is using in SnowieCaster code for update_subscriptions method"""
	NO_DATA = 1
	NONE_DATA = 2


class SubscriptionUpdater(ABC):
	"""Abstract wrapper around SnowieCaster class for typization"""
	@abstractmethod
	async def _update_subscriptions(self, channel: str):
		"""Abstract method for typization"""
		pass


class AbstractBackend(ABC):
	"""Abstract class that created for multibackend support"""
	def _start(self, *args, **kwargs) -> None:
		raise NotImplementedError()

	def _stop(self) -> None:
		raise NotImplementedError()

	@abstractmethod
	async def _astart(self, *args, **kwargs) -> None:
		pass

	@abstractmethod
	async def _astop(self) -> None:
		pass

	@abstractmethod
	async def _apublish(self, channel: str, message: Any, *args, **kwargs) -> None:
		pass

	@abstractmethod
	async def aget_next(self, channel: str) -> AsyncIterator[Any]:
		pass


class ISyncBackend(AbstractBackend):
	"""Class that wraps sync methods to async"""

	async def _astart(self, *args, **kwargs) -> None:
		return self._start(*args, **kwargs)

	async def _astop(self) -> None:
		return self._stop()



"""
Memory backend class
"""
#pylint: disable=W0311, R0903, W0221, W0201

from asyncio import Queue
from typing import Any, Dict, AsyncIterator
from ...abstract import ISyncBackend, Results


class MemoryBackend(ISyncBackend):
	"""This class allows you to use pub/sub in-memory"""
	def _check_instance_vars(self):
		"""This method checks is class vars are valid"""
		assert hasattr(self, 'store'), 'store is not defined, mb u call start?'
		assert isinstance(self.store, dict), \
		"store's datatype is not Dict[str, Queue]"

	def _start(self, store: Dict[str, Queue] = None):
		self.store: Dict[str, Queue] = {} if store is None else store
		self._check_instance_vars()

	def _stop(self):
		self.store = {}

	async def _apublish(self, channel: str, message: Any) -> None:
		self._check_instance_vars()
		assert isinstance(channel, str), "channel's datatype is not str"
		if channel not in self.store:
			self.store[channel] = Queue()
		sub = self.store[channel]
		assert isinstance(sub, Queue), \
		"store's datatype is not Dict[str, Queue]"
		await sub.put(message)

	async def aget_next(self, channel: str) -> Any:
		self._check_instance_vars()
		assert isinstance(channel, str), "channel's datatype is not str"
		if channel not in self.store:
			self.store[channel] = Queue()
		channel_queue = self.store[channel]
		if not isinstance(channel_queue, Queue) or channel_queue.empty():
			return Results.NO_DATA
		else:
			return await channel_queue.get()
		
